#ifndef ENGINE_H_
#define ENGINE_H_

#include <QObject>
#include <QDateTime>
#include "Room.h"
#include "WindowManager.h"
#include "DeviceManager.h"
#include "PasswordDialog.h"

class QTimer;
class Clock;
class Configuration;
class CommunicationManager;
class Meeting;
class UIManager;

//! BusinessLogic class. Contains all the business logic of the application.
/*!
 * BusinessLogic class. Contains all the business logic of the application. This class handles
 * the logic of the application according received calls to the slots. This class also maintains
 * the instances of managers, timers and its own set of meeting list. The managers handles
 * their tasks in more detailed manner.
 */
class Engine : public QObject
{
	Q_OBJECT

public:
	//! Constructor.
	/*!
	 * Constructor to initialize an Engine instance.
	 */
	Engine();
	//! Destructor.
	virtual ~Engine();
	//! Gets default room of the application.
	/*!
	 * Gets default room of the application.
	 * \return Pointer to the default Room instance.
	 */
	Room* defaultRoom();

signals:

	void roomStatusChanged( Room::Status aStatus, QTime aUntil );

private slots:
	//! Slot. Closes the application.
	/*!
	 * Slot. Closes the application.
	 */
	void closeApplication();
	//! Slot. Checks actual availability information of the specified room.
	/*!
	 * Slot. Checks actual availability information of the specified room.
	 * \param aRoom The room which availability information is needed.
	 */
	void roomStatusInfoNeeded( Room *aRoom );
	//! Slot. Asks the communication to fetch new meeting data.
	/*!
	 * Slot. Asks the communication to fetch new meeting data.
	 * \param aCurrentRoom The current room.
	 */
	void shownWeekChanged( QDate aDate );
	//! Slot. Handles errors.
	/*!
	 * Slot. Handles errors and informs the UI by emitting the error() signal with the message in
	 * parameter.
	 * \param aCode The error code.
	 * \param aAddInfo Possible addition info.
	 */
	void errorHandler( int aCode, const QString &aAddInfo = "" );
	//! Slot. Saves fetched meetings to the current instance's local storage.
	/*!
	 * Slot. Saves fetched meetings to the current instance's local storage. Meetings are soted in a
	 * private QList, it is iterated through and signals are emitted if there is any new, updated or
	 * deleted meeting.
	 * \param aMeetings The list of freshly fetched meetings.
	 */
	void meetingsFetched( const QList<Meeting*>& );
	//! Slot. Checks the availability of all the rooms.
	/*!
	 * Slot. Checks the availability of all the rooms by iterating through the current object's local
	 * room storage and calling the roomStatusInfoNeeded() separately on each of them.
	 */
	void checkStatusOfAllRooms();
	//! Slot for receiving the failure event of operation mode changing.
	/*!
	 * Slot. Receives the failure event of operation mode changing.
	 */
	void changeModeFailed();
	//! Slot for receiving the cancel event of the progress bar.
	/*!
	 *  Receives the cancel event of the progress bar when meeting details requested.
	 */
	void fetchMeetingDetails( Meeting *aMeeting );
	void cancelFetchMeetingDetails();
	
	void handleViewEvent();
	void previousViewRestored();
	
	//! Slot for dialog activation signal.
	/*!
	 * This slot is used to inform that dialog is activated. It stops
	 * the idle time counter so screensaver is not activated while the
	 * dialog is displayed.
	 */
	void dialogActivated();
	//! Slot for dialog deactivation signal.
	/*!
	 * This slot is used to inform that dialog is deactivated. It restarts
	 * the idle time counter so that the screensaver is being activated again
	 * as needed.
	 */
	void dialogDeactivated();
	
	void stopIdleTimeCounter();
	void startIdleTimeCounter();

	void changeDeviceMode( bool aChange );
	
	void currentRoomChanged( Room *aRoom );
	
private:
	// Make the UIManager as friendly class so it can connect to private slots.
	friend class UIManager;
	
	//! Provides the index of the Meeting instance which is at the specified time.
	/*!
	 * Provides the index of the Meeting instance which is at the specified time. If there are
	 * overlapping meetings then returns one of then undetetministically.
	 * \param aRoom The room which meetings are looked through.
	 * \param aAt Date and time when the meeting is already going.
	 * \return Index of the meeting if found; otherwise, -1.
	 */
	int indexOfMeetingAt( Room *aRoom, QDateTime aAt );
	//! Provides the index of the Meeting instance which is starts the closest to the specified time.
	/*!
	 * Provides the index of the Meeting instance which is starts the closest to the specified time.
	 * If there are overlapping meetings then returns one of then undetetministically.
	 * \param aRoom The room which meetings are looked through.
	 * \param aAt Date and time when the meeting is not yet started.
	 * \return Index of the meeting if found; otherwise, -1.
	 */
	int indexOfMeetingAfter( Room *aRoom, QDateTime aAfter );
	//! Slot. Fetches meetings from the server.
	/*!
	 * Slot. Fetches meetings from the server, exact parameters are specified in the parameter list.
	 * \param aFrom Time from when the meetings need to be fetched.
	 * \param aUntil Time until when the meetings need to be fetched.
	 * \param aIn The room which meetings need to be fetched.
	 */
	void fetchMeetings( const QDateTime &aFrom, const QDateTime &aUntil, const Room *aIn );
	//! Initialize configuration package.
	/*!
	 * This method initializes configuration classes and
	 * connects signals from and to the engine.
	 */
	void initConfiguration();
	//! Initialize device package.
	/*!
	 * This method initializes device manager and
	 * connects signals from and to the engine.
	 */
	void initDevice();
	//! Initialize communication package.
	/*!
	 * This method initializes the communication manager and
	 * connects signals from and to the engine.
	 */
	void initCommunication();
	//! Initialize user interface package.
	/*!
	 * This method initializes the user interface and
	 * connects signals from and to the engine. This method
	 * makes the window manager visible and shows weekly
	 * view as the first view.
	 */
	void initUserInterface();
	//! Connects signal between objects.
	/*!
	 * Signals that could not be connected while initializing different
	 * packages are connected here.
	 */
	void connectSignals();
	
	bool isMeetingInList(const QList<Meeting*> &aList, const Meeting *aMeeting);

private:
	static QTime endOfTheDay;

	WindowManager *iWindowManager;
	
	QTimer *iIdleTimeCounter;
	Clock *iClock;
	Configuration *iConfiguration;
	CommunicationManager *iCommunication;
	DeviceManager *iDevice;
	UIManager *iUIManager;

	QTimer *iAutoRefresh;

	QList<Meeting*> iMeetings;
	
	Room *iCurrentRoom;
};

#endif /*ENGINE_H_*/
