#ifndef UIMANAGER_H_
#define UIMANAGER_H_

#include <QObject>
#include <QList>

#include "Meeting.h"
#include "DeviceManager.h"
#include "PasswordDialog.h"

class Engine;
class WindowManager;
class WeeklyViewWidget;
class SettingsView;
class RoomStatusIndicatorWidget;
class ProgressBar;
class MeetingInfoDialog;
class CommunicationManager;
class QDateTime;

//! User Interface manager class.
/*!
 * User Interface manager class that is responsible for handling
 * UI actions. It creates the needed views and connects signals
 * accordingly. All interactions with the UI is handled by this
 * class. The Engine class handles the basic logic and does not
 * know or care about the UI and this class does not know or care
 * about the basic logic only UI related actions (signals, events etc.).
 */
class UIManager : public QObject
{
	Q_OBJECT
	
public:
	UIManager( Engine *aEngine, WindowManager *aWindowManager );
	virtual ~UIManager();
	
	//! Connects Device Managers signals.
	/*!
	 * This method connects Device Managers signals directly to UI
	 * components or to it selft.
	 */
	void connectDeviceManager( DeviceManager *aDeviceManager );
	//! Connects Communication Managers signals.
	/*!
	 * This method connects Communication Managers signals directly to UI
	 * components or to it selft.
	 */
	void connectCommunicationManager( CommunicationManager *aCommunicationManager );
	//! Shows the main view.
	/*!
	 * Makes the main view visible trough WindowManager.
	 */
	void showMainView();

signals:
	
public slots:

	//! Handles setting view request.
	/*!
	 * Handles request to show settings view. Makes the
	 * view visible and stops the idle time counter.
	 */
	void settingsViewRequest();
	//! Handles setting views ok clicked.
	/*!
	 * Handles the setting views Ok button clicked
	 * signal. Sets the weekly view visible and starts
	 * the idle time counter.
	 */
	void settingsOkClicked();
	//! Handles room status indicator view request.
	/*!
	 * Handles the request to show room status indicator
	 * view. Sets the view visible and stops the idle
	 * time counter. The WindowManager handles restoring
	 * what ever view was previously visible.
	 */
	void roomStatusIndicatorRequested();
	//! Handles previousViewRestored signal.
	/*!
	 * Handles the restoring of previous view. This is usually
	 * signaled by WindowManager when room status indicator
	 * view is being hidden.
	 */
	void previousViewRestored();
	//! Handle change mode order.
	/*!
	 * Handles change mode order. Displays the password query dialog
	 * and waits for its response.
	 */
	void changeModeOrdered( DeviceManager::OperationMode aMode );
	//! Handles select room change.
	/*!
	 * Handles the changing of currently select room. Engine is requested
	 * to start fetching new meetings for currently shown week.
	 */
	void currentRoomChanged( Room *aRoom );
	
private slots:

	void meetingsFetched( const QList<Meeting*> &aMeetings );
	void meetingDetailsFetched( Meeting &aDetailedMeeting );
	void progressBarCancelled();
	void updateTime( QDateTime aDateTime );
	void passwordEntered( PasswordDialog::PasswordStatus aStatus );
	void showMeetingProgressBar( Meeting *aMeeting );
	void updateProgressBarText( const QString &aText );
	void hideProgressBar();

private:
	
	void createWeeklyView();
	void createSettingsView();
	void createRoomStatusIndicator();
	void createPasswordDialog();
	void createProgressBar();
	void createMeetingInfoDialog();
	
private:
	Engine *iEngine;
	WindowManager *iWindowManager;
	
	WeeklyViewWidget *iWeeklyView;
	SettingsView *iSettingsView;
	RoomStatusIndicatorWidget *iRoomStatusIndicator;
	PasswordDialog *iPasswordDialog;
	ProgressBar *iProgressBar;
	MeetingInfoDialog *iMeetingInfo;
};

#endif /*UIMANAGER_H_*/
