#ifndef DATETIMESETTINGS_H_
#define DATETIMESETTINGS_H_

#include <QDateTime>
#include <QString>

#define DAYS_IN_A_WEEK 7

class DateTimeSettings
{
public:

	/*!
	 * An enumeration for the weekdays to be used.
	 */
	typedef enum 
	{
		Sunday = 0,
		Monday,
		Tuesday,
		Wednesday,
		Thursday,
		Friday,
		Saturday
	} weekDay;

	/*!
	 * Overloaded.
	 * \param aStandardTimeChange The date and time when the standard time is to be used. The year part and the milliseconds are omitted.
	 * \param aDaylightTimeChange The date and time when the daylight time is to be used. The year part and the milliseconds are omitted.
	 * \param aDayOfWeek The day of the week when the time is to be used.
	 */
	DateTimeSettings(const QDateTime &aStandardTimeChange, const QDateTime &aDaylightTimeChange, weekDay aDayOfWeek);

	//! Copy constructor.
	/*!
	 * Overloaded. Constructor to initialize a DateTimeSettings instance.
	 * \param aOther DateTimeSettings class to be copied.
	 */
	DateTimeSettings(const DateTimeSettings &aOther);
	//! Destructor
	virtual ~DateTimeSettings();

	/*!
	 * \return The Date/time when the standard time is to be switched into use
	 */
	QDateTime STDDateTime();

	/*!
	 * \return The Date/time when the daylight saving time is to be switched into use
	 */
	QDateTime DSTDateTime();

	/*!
	 * \return The time as a string (hh:mm:ss) of the moment the standard time is to be switched into use
	 */
	QString STDTime();

	/*!
	 * \return The month as a string (MM) of the moment the standard time is to be switched into use
	 */
	QString STDMonth();

	/*!
	 * \return The day as a string (dd) of the moment the standard time is to be switched into use
	 */
	QString STDDay();

	/*!
	 * \return The time as a string (hh:mm:ss) of the moment the daylight saving time is to be switched into use
	 */
	QString DSTTime();

	/*!
	 * \return The month as a string (MM) of the moment the daylight saving time is to be switched into use
	 */
	QString DSTMonth();

	/*!
	 * \return The day as a string (dd) of the moment the daylight saving time is to be switched into use
	 */
	QString DSTDay();

	/*!
	 * Sets the time/date of the moment the daylight saving time is to be switched into use
	 */
	void setDSTDateTime(const QDateTime &aDaylightSaveTimeChange);

	/*!
	 * Sets the time/date of the moment the standard time is to be switched into use
	 */
	void setSTDDateTime(const QDateTime &aStandardTimeChange);

	/*!
	 * Sets the appropriate day of the week the time switch is carried out.
	 * \see DateTimeSettings::weekDay
	 */
	void setSwitchDay(DateTimeSettings::weekDay aDay);

	/*!
	 * \return The appropriate day of the week the time switch is carried out.
	 */
	QString switchDay();

private:
	QDateTime iStandardTimeChangeMoment;
	QDateTime iDaylightSaveTimeChangeMoment;
	weekDay iDayOfWeek;

	/*!
	 * Contains the weekday strings that are passed to the exchange server when
	 * defining the timezone variables
	 */
	static const QString weekDays[DAYS_IN_A_WEEK];
};


#endif /*DATETIMESETTINGS_H_*/
